using System;
using System.Data;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Drawing;
using System.Windows.Forms;
using System.Drawing;

using Syncfusion.Windows.Forms.Tools;

namespace WindowsApplication1
{
	public class MyTextBox : TextBox 
	{
		public MyTextBox() : base()
		{
		}

		public bool keyPressed = false;

		//pass the enter key back to the grid....
		//and trigger change event on other keystrokes...
		protected override bool ProcessCmdKey(ref Message msg,Keys keyData)
		{
			if(keyData == Keys.Enter || keyData == Keys.Down )
			{
				this.OnKeyDown(new KeyEventArgs(keyData));  
				return true;
			}
			else if(keyData == Keys.Escape)
				return true;

			if(msg.Msg == 0x100 && keyData != Keys.Tab)//keydown...
			{
				this.OnTextChanged(EventArgs.Empty);
			}
			return base.ProcessCmdKey(ref msg, keyData);
		}
	}


	#region the cell model class
	public class TextBoxAutoCompleteCellModel : GridStaticCellModel
	{
		public Syncfusion.Windows.Forms.Tools.AutoComplete autoComplete1;

		public TextBoxAutoCompleteCellModel (GridModel grid)
			: base(grid)
		{	 
			this.autoComplete1 = new Syncfusion.Windows.Forms.Tools.AutoComplete();
			 

		}

		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new TextBoxAutoCompleteCellRenderer(control, this);
		}
	}
	#endregion

	#region the cell renderer class

	public class TextBoxAutoCompleteCellRenderer : GridStaticCellRenderer
	{
		private MyTextBox  myTextBox;
		public Syncfusion.Windows.Forms.Tools.AutoComplete autoComplete1;

		public TextBoxAutoCompleteCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{

			myTextBox = new MyTextBox();
			
			TextBoxAutoCompleteCellModel model = (TextBoxAutoCompleteCellModel) this.Model;
			//this.autoComplete1 = new Syncfusion.Windows.Forms.Tools.AutoComplete();
			this.autoComplete1 = model.autoComplete1;
			
			this.autoComplete1.SetAutoComplete(this.myTextBox, Syncfusion.Windows.Forms.Tools.AutoCompleteModes.AutoSuggest);

			this.autoComplete1.AccessibleDescription = null;
			this.autoComplete1.AccessibleName = null;
			this.autoComplete1.AccessibleRole = System.Windows.Forms.AccessibleRole.None;
			this.autoComplete1.AdjustHeightToItemCount = true;
			this.autoComplete1.AutoAddItem = false;
			this.autoComplete1.AutoSerialize = true;
			this.autoComplete1.CategoryName = "";

			this.autoComplete1.PreferredHeight = 200;
			this.autoComplete1.SelectedIndex = -1;
			this.autoComplete1.SelectedValue = "";
			this.autoComplete1.ShowCloseButton = true;
			this.autoComplete1.ShowGripper = true;
			
			myTextBox.TextChanged += new EventHandler(myTextBox_TextChanged);
			
			FixControlParent(myTextBox);
			this.SupportsFocusControl = true;
		}

		void FixControlParent(Control control)
		{
			if (control.Parent != Grid || !control.Visible)
			{
				control.Location = new Point(-1000, -1000);
				control.CausesValidation = false;
				control.Anchor = AnchorStyles.None;
				control.Dock = DockStyle.None;
				control.Parent = Grid.GetWindow();
				control.Visible = true;
				SetControl(control);
			}
		}

		protected override void Dispose(bool disposing)
		{
			myTextBox.TextChanged -= new EventHandler(myTextBox_TextChanged);			
			base.Dispose(disposing);
		}

		#region usual renderer overrides

		//handle drawing the cell
		protected override void OnDraw(System.Drawing.Graphics g, System.Drawing.Rectangle clientRectangle, int rowIndex, int colIndex, Syncfusion.Windows.Forms.Grid.GridStyleInfo style)
		{
			if (Grid.CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && Grid.CurrentCell.IsEditing)
			{
				myTextBox.Size = clientRectangle.Size;
				myTextBox.Location = clientRectangle.Location;
				myTextBox.Show();
			}
			else
			{
				style.TextMargins.Left = 3; //avoid the little jump...
				base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
			}
		}

		//set the value into the cell control & initialize it
		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			// Immeditaly switch into editing mode when cell is initialized.
			GridStyleInfo style = Grid.Model[rowIndex, colIndex];
			
			try
			{
				AutoCompleteDataColumnInfo info = style.Tag as AutoCompleteDataColumnInfo;
            
				object datasource = style.DataSource;
				if( datasource == null)
				{
					DataTable dt = new DataTable("AutoCompleteTable");
					dt.Columns.Add(new DataColumn("DisplayField"));   
					dt.Rows.Add(new object[]{string.Empty});   
					dt.AcceptChanges(); 
					this.CurrentStyle.DataSource = style.DataSource = dt;  
				}
				if( info == null )
				{
					info = new AutoCompleteDataColumnInfo("DisplayField",200,true);
					this.CurrentStyle.Tag = style.Tag = info;
					  
				}

				myTextBox.Text = style.CellValue.ToString();
				this.autoComplete1.SetAutoComplete(this.myTextBox,AutoCompleteModes.AutoSuggest);   
				this.autoComplete1.DataSource = style.DataSource;

				autoComplete1.Columns.Clear();
				autoComplete1.Columns.Add(style.Tag as AutoCompleteDataColumnInfo);
			}
			catch (Exception ex)
			{
				Syncfusion.Diagnostics.TraceUtil.TraceExceptionCatched(ex);
			}			
			base.OnInitialize(rowIndex, colIndex);
		}

		//save the changes from the cell control to the grid cell
		protected override bool OnSaveChanges()
		{
			if (CurrentCell.IsModified)
			{
				GridStyleInfo style = Grid.Model[this.RowIndex, this.ColIndex];
				style.CellValue = this.myTextBox.Text;
				return true;
			}
			return false;
		}
		#endregion	

		#region Click stuff
		//Simulate a click to place focus where user clicked in the control
		protected override void OnClick(int rowIndex, int colIndex, MouseEventArgs e)
		{
			base.OnClick(rowIndex, colIndex, e);
			if (e.Button == MouseButtons.Left)
			{
				CurrentCell.BeginEdit(true);
				//CurrentCell.HasControlFocus = true;
				//CurrentCell.Refresh();

				ClickControl();
			}
		}


		private Timer t;
		private void ClickControl()
		{
			t = new Timer();
			t.Interval = 20;
			t.Tick += new EventHandler(click);
			t.Start();
		}

		private void click(object sender, EventArgs e)
		{
			t.Stop();
			t.Tick -= new EventHandler(click);
			Point p = this.myTextBox.PointToClient(Control.MousePosition);
			ActiveXSnapshot.FakeLeftMouseClick(this.myTextBox, p);
			t.Dispose();
			t = null;
		}
		#endregion

		
		protected override bool OnValidate()
		{
			AutoCompleteDataColumnInfo info = this.CurrentStyle.Tag as AutoCompleteDataColumnInfo;
            
			object datasource = this.CurrentStyle.DataSource;
		
			if(info != null && datasource != null && this.myTextBox.Text != string.Empty)
			{
				DataTable dt = this.CurrentStyle.DataSource as DataTable;  
				string filter = string.Format("[{0}] = '{1}'",info.ColumnName,this.myTextBox.Text );

				DataRow[] row = dt.Select(filter);   
				if( row.Length == 0)
				{

					DataRow dr = dt.NewRow();   
					dr[info.ColumnName] = this.myTextBox.Text;
					dt.Rows.Add(dr); 
					dt.AcceptChanges(); 
			
					this.autoComplete1.SetAutoComplete(this.myTextBox,AutoCompleteModes.AutoSuggest);   
					this.autoComplete1.DataSource = dt;   
				}
			}
			return base.OnValidate ();
		}

		
		//handle initial keystroke on inactive cell, passing it to the datetimepicker
		protected override void OnKeyPress(System.Windows.Forms.KeyPressEventArgs e)
		{
			if(!myTextBox.Focused)
			{
				myTextBox.Focus();
				SendKeys.Send( new string(e.KeyChar, 1) );
			}
			base.OnKeyPress(e);
		}

		private void myTextBox_TextChanged(object sender, EventArgs e)
		{
			// Raise CurrentCellChanging event and set CurrentCell.Modified = true
			CurrentCell.IsModified = true;
			if (this.InitalizedAt(RowIndex, ColIndex)
				&& base.NotifyCurrentCellChanging())
			{
				CurrentCell.IsModified = true;
				ControlText= myTextBox.Text; 
				base.NotifyCurrentCellChanged();
				
			}
		}

		
		
	}

	#endregion
}